#include "WrappedTrackingSystem.hpp"

#include <iostream>
#include <limits>

using namespace std;

#ifdef BUILD_WITH_PRIVATE_SDK
    #define TrackingSystem TrackingSystemPrivate
#endif

WrappedTrackingSystem::WrappedTrackingSystem( uint32 timeout,
                                              bool   allowSimulator )
    : atracsys::TrackingSystem( timeout, allowSimulator )
{}

atracsys::Status WrappedTrackingSystem::initialise( const string& fileName )
{
    return TrackingSystem::initialise( fileName );
}

void WrappedTrackingSystem::streamLastError()
{
    atracsys::Status retCode( TrackingSystem::streamLastError() );


    if ( retCode != atracsys::Status::Ok )
    {
        cerr << __FILE__ << ": " << __LINE__
             << " atracsys::TrackingSystem::streamLastError returned status "
             << uint32( retCode ) << endl;
    }
}

vector< atracsys::DeviceInfo > WrappedTrackingSystem::getEnumeratedDevices()
const
{
    vector< atracsys::DeviceInfo > result;

    atracsys::Status retCode( atracsys::TrackingSystem::getEnumeratedDevices(
                                  result ) );

    if ( retCode != atracsys::Status::Ok )
    {
        cerr << __FILE__ << ": " << __LINE__
             <<
        "atracsys::TrackingSystem::getEnumeratedDevices returned status "
             << uint32( retCode ) << endl;
        result.clear();
    }

    return result;
}

vector< atracsys::DeviceOption > WrappedTrackingSystem::getEnumeratedOptions(
    uint64 sn ) const
{
    vector< atracsys::DeviceOption > result;

    atracsys::Status retCode( atracsys::TrackingSystem::getEnumeratedOptions(
                                  sn, result ) );

    if ( retCode != atracsys::Status::Ok )
    {
        cerr << __FILE__ << ": " << __LINE__
             <<
        "atracsys::TrackingSystem::getEnumeratedOptions returned status "
             << uint32( retCode ) << endl;
        result.clear();
    }

    return result;
}

vector< atracsys::DeviceOption > WrappedTrackingSystem::getEnumeratedOptions()
const
{
    vector< atracsys::DeviceOption > result;

    atracsys::Status retCode( atracsys::TrackingSystem::getEnumeratedOptions(
                                  result ) );

    if ( retCode != atracsys::Status::Ok )
    {
        cerr << __FILE__ << ": " << __LINE__
             <<
        "atracsys::TrackingSystem::getEnumeratedOptions returned status "
             << uint32( retCode ) << endl;
        result.clear();
    }

    return result;
}

// -----------------------------------------------------------------------------
//
//                             Integer option getters
//
// -----------------------------------------------------------------------------

int32 WrappedTrackingSystem::getIntOption( uint64          serialNbr,
                                           uint32          optId,
                                           ftkOptionGetter what ) const
{
    int32 result;

    atracsys::Status retCode( atracsys::TrackingSystem::getIntOption(
                                  serialNbr, optId, what, result ) );

    if ( retCode != atracsys::Status::Ok )
    {
        cerr << __FILE__ << ": " << __LINE__
             << " atracsys::TrackingSystem::getIntOption returned status "
             << uint32( retCode ) << endl;
        result = numeric_limits< int32 >::min();
    }

    return result;
}

int32 WrappedTrackingSystem::getIntOption( uint64 serialNbr, uint32 optId ) const
{
    int32 result;

    atracsys::Status retCode( atracsys::TrackingSystem::getIntOption(
                                  serialNbr, optId, result ) );

    if ( retCode != atracsys::Status::Ok )
    {
        cerr << __FILE__ << ": " << __LINE__
             << " atracsys::TrackingSystem::getIntOption returned status "
             << uint32( retCode ) << endl;
        result = numeric_limits< int32 >::min();
    }

    return result;
}

int32 WrappedTrackingSystem::getIntOption( uint32 optId, ftkOptionGetter what ) const
{
    int32 result;

    atracsys::Status retCode( atracsys::TrackingSystem::getIntOption(
                                  optId, what, result ) );

    if ( retCode != atracsys::Status::Ok )
    {
        cerr << __FILE__ << ": " << __LINE__
             << " atracsys::TrackingSystem::getIntOption returned status "
             << uint32( retCode ) << endl;
        result = numeric_limits< int32 >::min();
    }

    return result;
}

int32 WrappedTrackingSystem::getIntOption( uint32 optId ) const
{
    int32 result;

    atracsys::Status retCode( atracsys::TrackingSystem::getIntOption(
                                  optId, result ) );

    if ( retCode != atracsys::Status::Ok )
    {
        cerr << __FILE__ << ": " << __LINE__
             << " atracsys::TrackingSystem::getIntOption returned status "
             << uint32( retCode ) << endl;
        result = numeric_limits< int32 >::min();
    }

    return result;
}

int32 WrappedTrackingSystem::getIntOption( uint64             serialNbr,
                                           const std::string& optId,
                                           ftkOptionGetter    what ) const
{
    int32 result;

    atracsys::Status retCode( atracsys::TrackingSystem::getIntOption(
                                  serialNbr, optId, what, result ) );

    if ( retCode != atracsys::Status::Ok )
    {
        cerr << __FILE__ << ": " << __LINE__
             << " atracsys::TrackingSystem::getIntOption returned status "
             << uint32( retCode ) << endl;
        result = numeric_limits< int32 >::min();
    }

    return result;
}

int32 WrappedTrackingSystem::getIntOption( uint64             serialNbr,
                                           const std::string& optId ) const
{
    int32 result;

    atracsys::Status retCode( atracsys::TrackingSystem::getIntOption(
                                  serialNbr, optId, result ) );

    if ( retCode != atracsys::Status::Ok )
    {
        cerr << __FILE__ << ": " << __LINE__
             << " atracsys::TrackingSystem::getIntOption returned status "
             << uint32( retCode ) << endl;
        result = numeric_limits< int32 >::min();
    }

    return result;
}

int32 WrappedTrackingSystem::getIntOption( const std::string& optId,
                                           ftkOptionGetter    what ) const
{
    int32 result;

    atracsys::Status retCode( atracsys::TrackingSystem::getIntOption(
                                  optId, what, result ) );

    if ( retCode != atracsys::Status::Ok )
    {
        cerr << __FILE__ << ": " << __LINE__
             << " atracsys::TrackingSystem::getIntOption returned status "
             << uint32( retCode ) << endl;
        result = numeric_limits< int32 >::min();
    }

    return result;
}

int32 WrappedTrackingSystem::getIntOption( const std::string& optId ) const
{
    int32 result;

    atracsys::Status retCode( atracsys::TrackingSystem::getIntOption(
                                  optId, result ) );

    if ( retCode != atracsys::Status::Ok )
    {
        cerr << __FILE__ << ": " << __LINE__
             << " atracsys::TrackingSystem::getIntOption returned status "
             << uint32( retCode ) << endl;
        result = numeric_limits< int32 >::min();
    }

    return result;
}

// -----------------------------------------------------------------------------
//
//                             Float option getters
//
// -----------------------------------------------------------------------------

float WrappedTrackingSystem::getFloatOption( uint64          serialNbr,
                                             uint32          optId,
                                             ftkOptionGetter what ) const
{
    float result;

    atracsys::Status retCode( atracsys::TrackingSystem::getFloatOption(
                                  serialNbr, optId, what, result ) );

    if ( retCode != atracsys::Status::Ok )
    {
        cerr << __FILE__ << ": " << __LINE__
             << " atracsys::TrackingSystem::getFloatOption returned status "
             << uint32( retCode ) << endl;
        result = numeric_limits< float >::min();
    }

    return result;
}

float WrappedTrackingSystem::getFloatOption( uint64 serialNbr, uint32 optId ) const
{
    float result;

    atracsys::Status retCode( atracsys::TrackingSystem::getFloatOption(
                                  serialNbr, optId, result ) );

    if ( retCode != atracsys::Status::Ok )
    {
        cerr << __FILE__ << ": " << __LINE__
             << " atracsys::TrackingSystem::getFloatOption returned status "
             << uint32( retCode ) << endl;
        result = numeric_limits< float >::min();
    }

    return result;
}

float WrappedTrackingSystem::getFloatOption( uint32          optId,
                                             ftkOptionGetter what ) const
{
    float result;

    atracsys::Status retCode( atracsys::TrackingSystem::getFloatOption(
                                  optId, what, result ) );

    if ( retCode != atracsys::Status::Ok )
    {
        cerr << __FILE__ << ": " << __LINE__
             << " atracsys::TrackingSystem::getFloatOption returned status "
             << uint32( retCode ) << endl;
        result = numeric_limits< float >::min();
    }

    return result;
}

float WrappedTrackingSystem::getFloatOption( uint32 optId ) const
{
    float result;

    atracsys::Status retCode( atracsys::TrackingSystem::getFloatOption(
                                  optId, result ) );

    if ( retCode != atracsys::Status::Ok )
    {
        cerr << __FILE__ << ": " << __LINE__
             << " atracsys::TrackingSystem::getFloatOption returned status "
             << uint32( retCode ) << endl;
        result = numeric_limits< float >::min();
    }

    return result;
}

float WrappedTrackingSystem::getFloatOption( uint64             serialNbr,
                                             const std::string& optId,
                                             ftkOptionGetter    what ) const
{
    float result;

    atracsys::Status retCode( atracsys::TrackingSystem::getFloatOption(
                                  serialNbr, optId, what, result ) );

    if ( retCode != atracsys::Status::Ok )
    {
        cerr << __FILE__ << ": " << __LINE__
             << " atracsys::TrackingSystem::getFloatOption returned status "
             << uint32( retCode ) << endl;
        result = numeric_limits< float >::min();
    }

    return result;
}

float WrappedTrackingSystem::getFloatOption( uint64             serialNbr,
                                             const std::string& optId ) const
{
    float result;

    atracsys::Status retCode( atracsys::TrackingSystem::getFloatOption(
                                  serialNbr, optId, result ) );

    if ( retCode != atracsys::Status::Ok )
    {
        cerr << __FILE__ << ": " << __LINE__
             << " atracsys::TrackingSystem::getFloatOption returned status "
             << uint32( retCode ) << endl;
        result = numeric_limits< float >::min();
    }

    return result;
}

float WrappedTrackingSystem::getFloatOption( const std::string& optId,
                                             ftkOptionGetter    what ) const
{
    float result;

    atracsys::Status retCode( atracsys::TrackingSystem::getFloatOption(
                                  optId, what, result ) );

    if ( retCode != atracsys::Status::Ok )
    {
        cerr << __FILE__ << ": " << __LINE__
             << " atracsys::TrackingSystem::getFloatOption returned status "
             << uint32( retCode ) << endl;
        result = numeric_limits< float >::min();
    }

    return result;
}

float WrappedTrackingSystem::getFloatOption( const std::string& optId ) const
{
    float result;

    atracsys::Status retCode( atracsys::TrackingSystem::getFloatOption(
                                  optId, result ) );

    if ( retCode != atracsys::Status::Ok )
    {
        cerr << __FILE__ << ": " << __LINE__
             << " atracsys::TrackingSystem::getFloatOption returned status "
             << uint32( retCode ) << endl;
        result = numeric_limits< float >::min();
    }

    return result;
}

// -----------------------------------------------------------------------------
//
//                             Binary option getters
//
// -----------------------------------------------------------------------------

string WrappedTrackingSystem::getDataOption( uint64 serialNbr, uint32 optId ) const
{
    string result;

    atracsys::Status retCode( atracsys::TrackingSystem::getDataOption(
                                  serialNbr, optId, result ) );

    if ( retCode != atracsys::Status::Ok )
    {
        cerr << __FILE__ << ": " << __LINE__
             << " atracsys::TrackingSystem::getDataOption returned status "
             << uint32( retCode ) << endl;
        result = "ERROR";
    }

    return result;
}

string WrappedTrackingSystem::getDataOption( uint32 optId ) const
{
    string result;

    atracsys::Status retCode( atracsys::TrackingSystem::getDataOption(
                                  optId, result ) );

    if ( retCode != atracsys::Status::Ok )
    {
        cerr << __FILE__ << ": " << __LINE__
             << " atracsys::TrackingSystem::getDataOption returned status "
             << uint32( retCode ) << endl;
        result = "ERROR";
    }

    return result;
}

string WrappedTrackingSystem::getDataOption( uint64             serialNbr,
                                             const std::string& optId ) const
{
    string result;

    atracsys::Status retCode( atracsys::TrackingSystem::getDataOption(
                                  serialNbr, optId, result ) );

    if ( retCode != atracsys::Status::Ok )
    {
        cerr << __FILE__ << ": " << __LINE__
             << " atracsys::TrackingSystem::getDataOption returned status "
             << uint32( retCode ) << endl;
        result = "ERROR";
    }

    return result;
}

string WrappedTrackingSystem::getDataOption( const std::string& optId ) const
{
    string result;

    atracsys::Status retCode( atracsys::TrackingSystem::getDataOption(
                                  optId, result ) );

    if ( retCode != atracsys::Status::Ok )
    {
        cerr << __FILE__ << ": " << __LINE__
             << " atracsys::TrackingSystem::getDataOption returned status "
             << uint32( retCode ) << endl;
        result = "ERROR";
    }

    return result;
}

